@@ -8,6 +8,7 @@ import shutil
 import subprocess
 import textwrap
 
+from ..constants import CMAKE_DEFAULT_EXECUTABLE
 from ..exceptions import SKBuildGeneratorNotFoundError
 from ..utils import push_dir
 
@@ -21,7 +22,8 @@ class CMakePlatform(object):
     Derived class should at least set :attr:`default_generators`.
     """
     def __init__(self):
-        self._default_generators = list()
+        self._default_generators = []
+        self.architecture = None
 
     @property
     def default_generators(self):
@@ -44,7 +46,7 @@ class CMakePlatform(object):
         if not os.path.exists(test_folder):
             os.makedirs(test_folder)
         with open("{:s}/{:s}".format(test_folder, "CMakeLists.txt"), "w") as f:
-            f.write("cmake_minimum_required(VERSION 2.8)\n")
+            f.write("cmake_minimum_required(VERSION 2.8.12)\n")
             f.write("PROJECT(compiler_test NONE)\n")
             for language in languages:
                 f.write("ENABLE_LANGUAGE({:s})\n".format(language))
@@ -65,11 +67,19 @@ class CMakePlatform(object):
 
         return CMakeGenerator(generator_name)
 
+    def get_generators(self, generator_name):
+        """Loop over generators and return all that match the given name.
+        """
+        return [default_generator
+                for default_generator in self.default_generators
+                if default_generator.name == generator_name]
+
     # TODO: this method name is not great.  Does anyone have a better idea for
     # renaming it?
     def get_best_generator(
-            self, generator_name=None, languages=("CXX", "C"), cleanup=True,
-            cmake_executable='cmake', cmake_args=()):
+            self, generator_name=None, skip_generator_test=False,
+            languages=("CXX", "C"), cleanup=True,
+            cmake_executable=CMAKE_DEFAULT_EXECUTABLE, cmake_args=(), architecture=None):
         """Loop over generators to find one that works by configuring
         and compiling a test project.
 
@@ -77,6 +87,11 @@ class CMakePlatform(object):
         instead of trying :attr:`default_generators`.
         :type generator_name: string or None
 
+        :param skip_generator_test: If set to True and if a generator name is \
+        specified, the generator test is skipped. If no generator_name is specified \
+        and the option is set to True, the first available generator is used.
+        :type skip_generator_test: bool
+
         :param languages: The languages you'll need for your project, in terms \
         that CMake recognizes.
         :type languages: tuple
@@ -108,17 +123,28 @@ class CMakePlatform(object):
             # Lookup CMakeGenerator by name. Doing this allow to get a
             # generator object with its ``env`` property appropriately
             # initialized.
+
+            # MSVC should be used in "-A arch" form
+            if architecture is not None:
+                self.architecture = architecture
+
+            # Support classic names for generators
+            generator_name, self.architecture = _parse_legacy_generator_name(generator_name, self.architecture)
+
+            candidate_generators = []
             for default_generator in self.default_generators:
                 if default_generator.name == generator_name:
-                    candidate_generators = [default_generator]
-                    break
+                    candidate_generators.append(default_generator)
             if not candidate_generators:
                 candidate_generators = [CMakeGenerator(generator_name)]
 
         self.write_test_cmakelist(languages)
 
-        working_generator = self.compile_test_cmakelist(
-            cmake_executable, candidate_generators, cmake_args)
+        if skip_generator_test:
+            working_generator = candidate_generators[0]
+        else:
+            working_generator = self.compile_test_cmakelist(
+                cmake_executable, candidate_generators, cmake_args)
 
         if working_generator is None:
             raise SKBuildGeneratorNotFoundError(textwrap.dedent(
@@ -160,14 +186,11 @@ class CMakePlatform(object):
         # Do not complain about unused CMake arguments
         cmake_args.insert(0, "--no-warn-unused-cli")
 
-        cmake_args_as_str = " ".join(
-            ['"{:s}"'.format(arg) for arg in cmake_args])
-
         def _generator_discovery_status_msg(_generator, suffix=""):
             outer = "-" * 80
             inner = ["-" * ((idx * 5) - 3) for idx in range(1, 8)]
             print(outer if suffix == "" else "\n".join(inner))
-            print("-- Trying \"%s\" generator%s" % (_generator.name, suffix))
+            print("-- Trying \"{}\" generator{}".format(_generator.description, suffix))
             print(outer if suffix != "" else "\n".join(inner[::-1]))
 
         for generator in candidate_generators:
@@ -181,10 +204,14 @@ class CMakePlatform(object):
             with push_dir('build', make_directory=True):
                 # call cmake to see if the compiler specified by this
                 # generator works for the specified languages
-                cmake_execution_string = '{:s} ../ -G "{:s}" {:s}'.format(
-                    cmake_exe_path, generator.name, cmake_args_as_str)
-                status = subprocess.call(
-                    cmake_execution_string, shell=True, env=generator.env)
+                cmd = [cmake_exe_path, '../', '-G', generator.name]
+                if generator.toolset:
+                    cmd.extend(['-T', generator.toolset])
+                if generator.architecture:
+                    cmd.extend(['-A', generator.architecture])
+                cmd.extend(cmake_args)
+
+                status = subprocess.call(cmd, env=generator.env)
 
             _generator_discovery_status_msg(
                 generator, " - %s" % ("success" if status == 0 else "failure"))
@@ -205,19 +232,66 @@ class CMakeGenerator(object):
     .. automethod:: __init__
     """
 
-    def __init__(self, name, env=None):
+    def __init__(self, name, env=None, toolset=None, arch=None):
         """Instantiate a generator object with the given ``name``.
 
         By default, ``os.environ`` is associated with the generator. Dictionary
         passed as ``env`` parameter will be merged with ``os.environ``. If an
         environment variable is set in both ``os.environ`` and ``env``, the
         variable in ``env`` is used.
+
+        Some CMake generators support a ``toolset`` specification to tell the native
+        build system how to choose a compiler.
         """
         self._generator_name = name
         self.env = dict(
             list(os.environ.items()) + list(env.items() if env else []))
+        self._generator_toolset = toolset
+        self._generator_architecture = arch
+        if arch is None:
+            description_arch = name
+        else:
+            description_arch = "{} {}".format(name, arch)
+        if toolset is None:
+            self._description = description_arch
+        else:
+            self._description = "{} {}".format(description_arch, toolset)
 
     @property
     def name(self):
         """Name of CMake generator."""
         return self._generator_name
+
+    @property
+    def toolset(self):
+        """Toolset specification associated with the CMake generator."""
+        return self._generator_toolset
+
+    @property
+    def architecture(self):
+        """Architecture associated with the CMake generator."""
+        return self._generator_architecture
+
+    @property
+    def description(self):
+        """Name of CMake generator with properties describing the environment (e.g toolset)"""
+        return self._description
+
+
+def _parse_legacy_generator_name(generator_name, arch):
+    # type: (str, str | None) -> tuple[str, str | None]
+    """
+    Support classic names for MSVC generators. Architecture is stripped from
+    the name and "arch" is replaced with the arch string if a legacy name is
+    given.
+    """
+
+    if generator_name.startswith("Visual Studio"):
+        if generator_name.endswith(" Win64"):
+            arch = "x64"
+            generator_name = generator_name[:-6]
+        elif generator_name.endswith(" ARM"):
+            arch = "ARM"
+            generator_name = generator_name[:-4]
+
+    return generator_name, arch
