@@ -4,11 +4,27 @@ import errno
 import os
 
 from collections import namedtuple
+import contextlib
 from contextlib import contextmanager
-from distutils import log as distutils_log
 from distutils.command.build_py import build_py as distutils_build_py
+from distutils.errors import DistutilsTemplateError
+from distutils.filelist import FileList
+from distutils.text_file import TextFile
 from functools import wraps
 
+try:
+    import setuptools.logging  # noqa: F401
+    import logging
+
+    distutils_log = logging.getLogger("skbuild")
+    distutils_log.setLevel(logging.INFO)
+    logging_module = True
+
+except ImportError:
+    from distutils import log as distutils_log
+
+    logging_module = False
+
 
 class ContextDecorator(object):
     """A base class or mixin that enables context managers to work as
@@ -150,17 +166,22 @@ class PythonModuleFinder(new_style(distutils_build_py)):
         return map(_strip_directory, modules)
 
     def check_module(self, module, module_file):
+        """Return True if ``module_file`` belongs to ``module``.
+        """
         if self.alternative_build_base is not None:
             updated_module_file = os.path.join(
                 self.alternative_build_base, module_file)
             if os.path.exists(updated_module_file):
                 module_file = updated_module_file
         if not os.path.isfile(module_file):
-            distutils_log.warn(
-                "file %s (for module %s) not found", module_file, module)
+            if logging_module:
+                distutils_log.warning(
+                    "file %s (for module %s) not found", module_file, module)
+            else:
+                distutils_log.warn(
+                    "file %s (for module %s) not found", module_file, module)
             return False
-        else:
-            return True
+        return True
 
 
 def to_platform_path(path):
@@ -176,11 +197,67 @@ def to_unix_path(path):
 
 @contextmanager
 def distribution_hide_listing(distribution):
-    """Given a ``distribution``, this context manager allow to
-    temporarily set distutils verbosity to 0."""
-    old_threshold = distutils_log._global_log.threshold
-    if (hasattr(distribution, "hide_listing")
-            and distribution.hide_listing):
-        distutils_log.set_verbosity(0)
-    yield
-    distutils_log.set_verbosity(old_threshold)
+    """Given a ``distribution``, this context manager temporarily
+    sets distutils threshold to WARN if ``--hide-listing`` argument
+    was provided.
+
+    It yields True if ``--hide-listing`` argument was provided.
+    """
+
+    hide_listing = hasattr(distribution, "hide_listing") and distribution.hide_listing
+
+    # pylint:disable=protected-access
+    if logging_module:
+        # Setuptools 60.2+, will always be on Python 3.6+
+        old_level = distutils_log.getEffectiveLevel()
+        if hide_listing:
+            distutils_log.setLevel(logging.WARNING)
+        try:
+            if hide_listing:
+                # The classic logger doesn't respond to set_threshold anymore,
+                # but it does log info and above to stdout, so let's hide that
+                with open(os.devnull, 'w') as f, contextlib.redirect_stdout(f):
+                    yield hide_listing
+            else:
+                yield hide_listing
+        finally:
+            distutils_log.setLevel(old_level)
+
+    else:
+        old_threshold = distutils_log._global_log.threshold
+        if hide_listing:
+            distutils_log.set_threshold(distutils_log.WARN)
+        try:
+            yield hide_listing
+        finally:
+            distutils_log.set_threshold(old_threshold)
+
+
+def parse_manifestin(template):
+    """This function parses template file (usually MANIFEST.in)"""
+    if not os.path.exists(template):
+        return []
+
+    template = TextFile(template, strip_comments=1, skip_blanks=1,
+                        join_lines=1, lstrip_ws=1, rstrip_ws=1,
+                        collapse_join=1)
+
+    file_list = FileList()
+    try:
+        while True:
+            line = template.readline()
+            if line is None:  # end of file
+                break
+
+            try:
+                file_list.process_template_line(line)
+            # the call above can raise a DistutilsTemplateError for
+            # malformed lines, or a ValueError from the lower-level
+            # convert_path function
+            except (DistutilsTemplateError, ValueError) as msg:
+                print("%s, line %d: %s" % (template.filename,
+                                           template.current_line,
+                                           msg))
+        return file_list.files
+    finally:
+        template.close()
